/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Certus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* GPIO Output Configuration */
public class ANPacket195
{
	public static final int LENGTH = 183;

	public static final int NMEA_FIX_BEHAVIOUR_NORMAL = 0;
	public static final int NMEA_FIX_BEHAVIOUR_ALWAYS_3D = 1;
	public static final int NMEA_FIX_BEHAVIOUR_MAXIMUM = 1;

	public static final String[] OUTPUT_RATE_STRINGS = new String[] {
		"Disabled",
		"0.1 Hz",
		"0.2 Hz",
		"0.5 Hz",
		"1 Hz",
		"2 Hz",
		"5 Hz",
		"10 Hz",
		"25 Hz",
		"50 Hz",
		"8 Hz"
		};
	public static final int RATE_DISABLED = 0;
	public static final int RATE_0o1Hz = 1;
	public static final int RATE_0o2Hz = 2;
	public static final int RATE_0o5Hz = 3;
	public static final int RATE_1Hz = 4;
	public static final int RATE_2Hz = 5;
	public static final int RATE_5Hz = 6;
	public static final int RATE_10Hz = 7;
	public static final int RATE_25Hz = 8;
	public static final int RATE_50Hz = 9;
	public static final int RATE_8Hz = 10;
	public static final int RATE_MAXIMUM = 10;

	public Boolean permanent;

	public Rates gpio1;		// GPIO port
	public Rates gpio3;		// Auxiliary port
	public Rates logging;
	public Rates data_ports[];

	public ANPacket195()
	{
	}

	public ANPacket195(ANPacket packet)
	{
		permanent = packet.data[0] != 0;
		gpio3 = new Rates();
		gpio3.readNew(packet.data, 1);

		gpio1 = new Rates();
		gpio1.readNew(packet.data, 27);

		logging = new Rates();
		logging.readNew(packet.data, 53);

		data_ports = new Rates[4];
		for (int i = 0; i < 4; ++i)
		{
			data_ports[i] = new Rates();
			data_ports[i].readNew(packet.data, 79 + i*26);
		}
	}

	public ANPacket toANPacket() throws IndexOutOfBoundsException
	{
		ANPacket packet;
		if (gpio1 == null || !gpio1.valid() || 
			gpio3 == null || !gpio3.valid() || 
			logging == null || !logging.valid() || 
			data_ports[0] == null || !data_ports[0].valid() ||
			data_ports[1] == null || !data_ports[1].valid() ||
			data_ports[2] == null || !data_ports[2].valid() ||
			data_ports[3] == null || !data_ports[3].valid())
		{
			throw new IndexOutOfBoundsException();
		}

		packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_GPIO_OUTPUT_CONFIGURATION);
		packet.data[0] = (byte) (permanent ? 1 : 0);
		gpio3.writeNew(packet.data, 1);
		gpio1.writeNew(packet.data, 27);
		logging.writeNew(packet.data, 53);
		for (int i = 0; i < 4; ++i)
		{
			data_ports[i].writeNew(packet.data, 79 + i*26);
		}

		return packet;
	}

	public static String rateToString(int rate)
	{
		if(rate < OUTPUT_RATE_STRINGS.length)
		{
			return OUTPUT_RATE_STRINGS[rate];
		}
		else
		{
			return "Unknown Rate";
		}
	}

	public static String nmeaFixBehaviourString(int val)
	{
		if (val == NMEA_FIX_BEHAVIOUR_NORMAL)
		{
			return "Normal";
		}
		else if (val == NMEA_FIX_BEHAVIOUR_ALWAYS_3D)
		{
			return "Always 3D Fix";
		}
		else
		{
			return "Unknown";
		}
	}

	public String nmeaFixBehaviourString()
	{
		if (gpio1 != null)
		{
			return nmeaFixBehaviourString(gpio1.nmeaFixBehaviour);
		}
		else
		{
			return "Unknown";
		}
	}

	public class Rates
	{
		public int nmeaFixBehaviour;
		public int gpzda;
		public int gpgga;
		public int gpvtg;
		public int gprmc;
		public int gphdt;
		public int gpgll;
		public int pashr;
		public int tss1;
		public int simrad;
		public int gprot;
		public int gphev;
		public int gpgsv;
		public int pfecatt;
		public int pfechve;
		public int gpgst;
		public int inputMode;
		public int outputMode;
		
		void readNew(byte data[], int offset)
		{
			inputMode 			= data[offset + 0] & 0xFF;
			outputMode	 		= data[offset + 1] & 0xFF;
			nmeaFixBehaviour 	= data[offset + 2] & 0xFF;

			gpzda 				= data[offset + 3] & 0xFF;
			gpgga 				= data[offset + 4] & 0xFF;
			gpvtg 				= data[offset + 5] & 0xFF;
			gprmc 				= data[offset + 6] & 0xFF;
			gphdt 				= data[offset + 7] & 0xFF;
			gpgll 				= data[offset + 8] & 0xFF;
			pashr 				= data[offset + 9] & 0xFF;
			tss1 				= data[offset + 10] & 0xFF;
			simrad 				= data[offset + 11] & 0xFF;
			gprot 				= data[offset + 12] & 0xFF;
			gphev 				= data[offset + 13] & 0xFF;
			gpgsv 				= data[offset + 14] & 0xFF;
			pfecatt 			= data[offset + 15] & 0xFF;
			pfechve 			= data[offset + 16] & 0xFF;
			gpgst 				= data[offset + 17] & 0xFF;
		}

		void writeNew(byte data[], int offset)
		{
			data[offset + 0] = (byte) (inputMode & 0xFF);
			data[offset + 1] = (byte) (outputMode  & 0xFF);
			data[offset + 2] = (byte) (nmeaFixBehaviour & 0xFF);

			data[offset + 3] = (byte) (gpzda & 0xFF);
			data[offset + 4] = (byte) (gpgga & 0xFF);
			data[offset + 5] = (byte) (gpvtg & 0xFF);
			data[offset + 6] = (byte) (gprmc & 0xFF);
			data[offset + 7] = (byte) (gphdt & 0xFF);
			data[offset + 8] = (byte) (gpgll & 0xFF);
			data[offset + 9] = (byte) (pashr & 0xFF);
			data[offset + 10] = (byte) (tss1 & 0xFF);
			data[offset + 11] = (byte) (simrad & 0xFF);
			data[offset + 12] = (byte) (gprot & 0xFF);
			data[offset + 13] = (byte) (gphev & 0xFF);
			data[offset + 14] = (byte) (gpgsv & 0xFF);
			data[offset + 15] = (byte) (pfecatt & 0xFF);
			data[offset + 16] = (byte) (pfechve & 0xFF);
			data[offset + 17] = (byte) (gpgst & 0xFF);
		}

		boolean valid()
		{
			if (nmeaFixBehaviour < 0 || nmeaFixBehaviour > NMEA_FIX_BEHAVIOUR_MAXIMUM) return false;
			if (gpzda < 0 || gpzda > RATE_MAXIMUM)		return false;
			if (gpgga < 0 || gpgga > RATE_MAXIMUM)		return false;
			if (gpvtg < 0 || gpvtg > RATE_MAXIMUM)		return false;
			if (gprmc < 0 || gprmc > RATE_MAXIMUM)		return false;
			if (gphdt < 0 || gphdt > RATE_MAXIMUM)		return false;
			if (gpgll < 0 || gpgll > RATE_MAXIMUM)		return false;
			if (pashr < 0 || pashr > RATE_MAXIMUM)		return false;
			if (tss1 < 0 || tss1 > RATE_MAXIMUM)		return false;
			if (simrad < 0 || simrad > RATE_MAXIMUM)	return false;
			if (gprot < 0 || gprot > RATE_MAXIMUM)		return false;
			if (gphev < 0 || gphev > RATE_MAXIMUM)		return false;
			if (gpgsv < 0 || gpgsv > RATE_MAXIMUM)		return false;
			if (pfecatt < 0 || pfecatt > RATE_MAXIMUM)	return false;
			if (pfechve < 0 || pfechve > RATE_MAXIMUM)	return false;
			if (gpgst < 0 || gpgst > RATE_MAXIMUM)		return false;

			return true;
		}
	}
}
